/* ATENCION *************** 
Esto es una forma de abreviar la apertura de un puerto serie en Linux
Fue tomado de cdigo de dominio pblico y se entrega slo como referencia
*/

#include <fcntl.h>
#include <stdio.h>
#include <termio.h>
#include <errno.h>

/************************************************************************

	set_up_comms

	This function sets up a serial port 

**************************************************************************/

int set_up_comms( char *device, int baud_i, char *parity )
{
	int ttyfd;
	struct termios settings;

	speed_t baud_rate;

#ifdef DEBUG
	fprintf( stderr, "opening %s\n", device );
#endif

	switch( baud_i )
	{
		case 110:
			baud_rate = B110;
			break;
		case 300:
			baud_rate = B300;
			break;
		case 600:
			baud_rate = B600;
			break;
		case 1200:
			baud_rate = B1200;
			break;
		case 2400:
			baud_rate = B2400;
			break;
		case 4800:
			baud_rate = B4800;
			break;
		case 9600: case 0:
			baud_rate = B9600;
			break;
		case 19200:
			baud_rate = B19200;
			break;
		case 38400:
			baud_rate = B38400;
			break;
		case 57600:
			baud_rate = B57600;
			break;
		case 115200:
			baud_rate = B115200;
			break;
		default:
			baud_rate = B9600;
			fprintf(stderr, "Unknown baud rate %d for %s.", baud_i, device);
	}



	if(( ttyfd = open( device,O_RDWR) ) < 0 )
	{
		fprintf( stderr, "Error opening device %s.   ", device );
			
		fprintf( stderr, "Error no. %d \n",errno );
		exit( 1 );	/* stop the program. This maybe should */
				/* be a bit kinder but will do for now.*/
	}

#ifdef DEBUG
	fprintf( stderr, "%s open\n", device );
#endif

	/* read your man page for the meaning of all this. # man termios */
	/* Its a bit to involved to comment here                         */


	cfsetispeed( &settings, baud_rate );/* Set the baud rate */
	cfsetospeed( &settings, baud_rate );

	settings.c_line = 0; 

	settings.c_iflag |= IGNBRK;
	settings.c_iflag |= IGNPAR;
	settings.c_iflag &=~ PARMRK;
	settings.c_iflag &=~ INPCK;
	settings.c_iflag &=~ ISTRIP;
	settings.c_iflag &=~ INLCR;
	settings.c_iflag &=~ IGNCR;
	settings.c_iflag &=~ ICRNL;
	settings.c_iflag &=~ IUCLC;
	settings.c_iflag &=~ IXON;
	settings.c_iflag |= IXANY;
	settings.c_iflag &=~ IXOFF;
	settings.c_iflag &=~ IMAXBEL;

	settings.c_oflag |= OPOST;
	settings.c_oflag &=~ OLCUC;
	settings.c_oflag &=~ ONLCR;
	settings.c_oflag &=~ OCRNL;
	settings.c_oflag |= ONOCR;
	settings.c_oflag &=~ ONLRET;
	settings.c_oflag &=~ OFILL;
	settings.c_oflag &=~ OFDEL;

	settings.c_cflag &=~ CSIZE;
	settings.c_cflag |= CS8;
	settings.c_cflag &=~ CSTOPB;
	settings.c_cflag |= CREAD;

	if( strncmp( parity, "none", 4 ) == 0 )
	{
		settings.c_cflag &=~ PARENB;
		settings.c_cflag &=~ PARODD;
	}
	else
	if( strncmp( parity, "even", 4 ) == 0 )
	{
		settings.c_cflag |= PARENB;
		settings.c_cflag &=~ PARODD;
	}
	else
	{
		settings.c_cflag |= PARENB;
		settings.c_cflag |= PARODD;
	}

	settings.c_cflag &=~ HUPCL;
	settings.c_cflag |= CLOCAL;
	settings.c_cflag &=~ CRTSCTS;

	settings.c_lflag &=~ ISIG;
	settings.c_lflag &=~ ICANON;
	settings.c_lflag &=~ ECHO;
	settings.c_lflag |= IEXTEN;

	settings.c_cc[VMIN] = 0;
	settings.c_cc[VTIME] = 0;

	if( tcsetattr( ttyfd, TCSANOW, &settings ) < 0 )
	{
		fprintf( stderr, "tcsetattr failed\n");
		exit( 1 );
	}


	return( ttyfd );
}









